@@ -9,19 +9,24 @@ import json
 import codecs
 import pprint
 import string
+import fnmatch
 
 
 PY3 = (sys.version_info[0] == 3)
 if PY3:
     unicode, basestring = str, str
 
-__version__ = '0.7.2'
+__version__ = '0.7.3'
 __author__ = 'Mahmoud Hashemi'
 __contact__ = 'mahmoudrhashemi@gmail.com'
 __url__ = 'https://github.com/mahmoud/ashes'
 __license__ = 'BSD'
 
 
+DEFAULT_EXTENSIONS = ('.dust', '.html', '.xml')
+DEFAULT_IGNORED_PATTERNS = ('.#*',)
+
+
 # need to add group for literals
 # switch to using word boundary for params section
 node_re = re.compile(r'({'
@@ -1724,6 +1729,17 @@ class Template(object):
         if not keep_source:
             self.source = None
 
+    @classmethod
+    def from_path(cls, path, name=None, encoding='utf-8', **kw):
+        abs_path = os.path.abspath(path)
+        if not os.path.isfile(abs_path):
+            raise TemplateNotFound(abs_path)
+        with codecs.open(abs_path, 'r', encoding) as f:
+            source = f.read()
+        if not name:
+            name = path
+        return cls(name=name, source=source, source_file=abs_path, **kw)
+
     def render(self, model, env=None):
         env = env or self.env
         rendered = []
@@ -1833,6 +1849,8 @@ class ParseError(AshesException):
 
 
 class BaseAshesEnv(object):
+    template_type = Template
+
     def __init__(self,
                  loaders=None,
                  helpers=None,
@@ -1908,11 +1926,25 @@ class BaseAshesEnv(object):
         self.templates[name] = template
         return
 
-    def register_source(self, name, source, **kw):
+    def register_path(self, path, name=None, **kw):
+        """\
+        Reads in, compiles, and registers a single template from a specific
+        path to a file containing the dust source code.
+        """
         kw['env'] = self
-        tmpl = Template(name, source, **kw)
-        self.register(tmpl)
-        return tmpl
+        ret = self.template_type.from_path(path=path, name=name, **kw)
+        self.register(ret)
+        return ret
+
+    def register_source(self, name, source, **kw):
+        """\
+        Compiles and registers a single template from source code
+        string. Assumes caller already decoded the source string.
+        """
+        kw['env'] = self
+        ret = self.template_type(name=name, source=source, **kw)
+        self.register(ret)
+        return ret
 
     def filter_ast(self, ast, optimize=True):
         if optimize:
@@ -1947,6 +1979,9 @@ class BaseAshesEnv(object):
             return chunk.set_error(tnf)
         return tmpl.render_chunk(chunk, context)
 
+    def __iter__(self):
+        return self.templates.itervalues()
+
 
 class AshesEnv(BaseAshesEnv):
     """
@@ -1972,20 +2007,43 @@ class AshesEnv(BaseAshesEnv):
         return super(AshesEnv, self).filter_ast(ast, optimize)
 
 
-DEFAULT_EXTENSIONS = ('.dust', '.html', '.xml')
+def iter_find_files(directory, patterns, ignored=None):
+    """\
+    Finds files under a `directory`, matching `patterns` using "glob"
+    syntax (e.g., "*.txt"). It's also possible to ignore patterns with
+    the `ignored` argument, which uses the same format as `patterns.
+
+    (from osutils.py in the boltons package)
+    """
+    if isinstance(patterns, basestring):
+        patterns = [patterns]
+    pats_re = re.compile('|'.join([fnmatch.translate(p) for p in patterns]))
+
+    if not ignored:
+        ignored = []
+    elif isinstance(ignored, basestring):
+        ignored = [ignored]
+    ign_re = re.compile('|'.join([fnmatch.translate(p) for p in ignored]))
+    for root, dirs, files in os.walk(directory):
+        for basename in files:
+            if pats_re.match(basename):
+                if ignored and ign_re.match(basename):
+                    continue
+                filename = os.path.join(root, basename)
+                yield filename
+    return
 
 
-def walk_ext_matches(path, exts=None):
+def walk_ext_matches(path, exts=None, ignored=None):
     if exts is None:
         exts = DEFAULT_EXTENSIONS
-    exts = list(['.' + e.lstrip('.') for e in exts])
-    matches = []
-    for root, _, filenames in os.walk(path):
-        for fn in filenames:
-            for ext in exts:
-                if fn.endswith(ext):
-                    matches.append(os.path.join(root, fn))
-    return matches
+    if ignored is None:
+        ignored = DEFAULT_IGNORED_PATTERNS
+    patterns = list(['*.' + e.lstrip('*.') for e in exts])
+
+    return sorted(iter_find_files(directory=path,
+                                  patterns=patterns,
+                                  ignored=ignored))
 
 
 class TemplatePathLoader(object):
@@ -2002,14 +2060,12 @@ class TemplatePathLoader(object):
         if not path.startswith(self.root_path):
             norm_path = os.path.join(self.root_path, norm_path)
         abs_path = os.path.abspath(norm_path)
-        if os.path.isfile(abs_path):
-            with codecs.open(abs_path, 'r', self.encoding) as f:
-                source = f.read()
-        else:
-            raise TemplateNotFound(path)
         template_name = os.path.relpath(abs_path, self.root_path)
-        template = Template(template_name, source, abs_path, env=env)
-        return template
+        template_type = env.template_type
+        return template_type.from_path(name=template_name,
+                                       path=abs_path,
+                                       encoding=self.encoding,
+                                       env=env)
 
     def load_all(self, env, exts=None, **kw):
         ret = []
